{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE QuasiQuotes #-}

module Main where

import Conduit
import Data.LanguageCodes (ISO639_1(..))
import Data.Text qualified as T
import HAL
import HAL (getMetadataWith)
import HAL.Client
import HAL.Doc
import HAL.Doc.Corpus (Corpus(..))
import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)
import Options.Applicative
import Protolude
import Servant.Client
import Tree


data CountParams = CountParams
  { cp_query :: [ T.Text ] }

data FetchParams = FetchParams
  { fp_query :: [ T.Text ] }

data Command =
    Count CountParams
  | Fetch FetchParams

countParams :: Parser Command
countParams = Count <$>
  (CountParams
  <$> many (strArgument (metavar "query")))

fetchParams :: Parser Command
fetchParams = Fetch <$>
  (FetchParams
  <$> many (strArgument (metavar "query")))

params :: Parser Command
params = subparser
  (command "count" (info countParams (progDesc "Count number of docs for a given query"))
  <> command "fetch" (info fetchParams (progDesc "Fetch docs for a given query")))

opts :: ParserInfo Command
opts = info (params <**> helper)
       (fullDesc
       <> progDesc "A program to test HAL"
       <> header "crawlerHAL-exe")

main :: IO ()
main = run =<< execParser opts
  -- res <- getMetadataWith (generateRequestByStructID "artificial intelligence" imt) (Just 0) (Just 55)
  -- case res of
  --   (Left  err) -> print err
  --   (Right val) -> print $ _docs val

run :: Command -> IO ()
run (Count (CountParams { cp_query })) = do
  res <- getMetadataWithC cp_query (Just 0) Nothing Nothing
  case res of
    Left err -> putText $ show err
    Right (cnt, _docsC) -> putText $ show cnt
run (Fetch (FetchParams { fp_query })) = do
  res <- getMetadataWithC fp_query (Just 0) Nothing Nothing
  case res of
    Left err -> putText $ show err
    Right (_cnt, docsC) -> do
      _ <- runConduit $
           docsC
        .| mapM_C printCorpus
        .| sinkList
      pure ()
    where
      printCorpus Corpus { .. } = do
        putText $ "docid: " <> _corpus_docid <> " [" <> (T.intercalate " " show _corpus_title_lang_map) <> "]"
        putText $ "  " <> (T.intercalate " " _corpus_abstract)
        putText $ "  " <> show _corpus_abstract_lang_map
        putText $ "  " <> show _corpus_original
        putText "------------"


-- data
-- yearReq = [text|
-- (language_t:en)
-- AND (producedDateY_i:2018
--      OR producedDateY_i:2019
--      OR producedDateY_i:2020
--      OR producedDateY_i:2021
--      OR producedDateY_i:2022)
-- AND (structId_i:302102
--     OR structId_i:469216
--     OR structId_i:6279
--     OR structId_i:224096
--     OR structId_i:144103
--     OR structId_i:497330
--     OR structId_i:1076752
--     OR structId_i:84538
--     OR structId_i:301262
--     OR structId_i:481355
--     OR structId_i:29212
--     OR structId_i:301442
--     OR structId_i:542824
--     OR structId_i:300362
--     OR structId_i:1048346
--     OR structId_i:352124)|]

-- imt :: [T.Text]
-- imt = [
--   "224096"
--   ,"144103"
--   ,"84538"
--   ,"300104"
--   ,"300362"
--   ,"301262"
--   ,"301442"
--   ,"301492"
--   ,"302102"
--   ,"421532"
--   ,"497330"
--   ,"352124"
--   ,"481355"
--   ,"469216"
--   ,"542824"
--   ,"6279"
--   ,"29212"
--   ]
